#include <stdlib.h>
#include <msp430x14x.h>
#ifndef _lcd_h
  #include "lcd.h"
#endif

unsigned char chars, row;

void wait(unsigned int i)               // delay function
{
  unsigned int j, k;
  for(k=0;k<235;k++)
  {
    for(j=0;j<i;j++);
  }
}

void wait_precise(unsigned int i)      // praeziser, wird nur fuer triac-zuendung benoetigt
{                                      // wartet i * 0,1ms
  unsigned int j, k;
  for(k=0;k<46;k++)
  {
    for(j=0;j<i;j++);
  }
}
/*
void LCDCursor( int onof )
{
  if( onof == 0 ) {
    P4OUT=0x0C;             // cursor aus
  }
  else {
    P4OUT=0x08;             // cursor aus
  }
  P5OUT=0x04;             // RS und R/W low, E high
  LCDClock();             // befehl ausfuehren
}
*/
void LCDClock(void)
{
  wait(1);
  P5OUT|=0x04;                          // clock-bit erst mal sicherheitshalber auf high
  wait(1);
  P5OUT&=~0x04;                         // und jetzt auf low
  wait(1);
}

void LCDClear(void)
{
  row=0;                                // zeilen- und zeichencounter resetten
  chars=0;
  P4OUT=0x01;                           // clear display
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
  wait(1);                              // clear display braucht was laenger
}

void LCDCmd( int cmd )
{
  P4OUT=cmd;                           // nur return home, ohne clear display
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
  wait(1);                              // return home braucht was laenger
}

void LCDHome(void)
{
  row=0;                                // zeilen- und zeichencounter resetten
  chars=0;
  P4OUT=0x02;                           // nur return home, ohne clear display
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
  wait(1);                              // return home braucht was laenger
}

void LCDPos(int pos)
{
      P4OUT= 0x80 + pos;  // in die zweite zeile springen
      P5OUT=0x04;         // RS und R/W low, E high
      LCDClock();         // befehl ausfuehren
}

void LCDOutc(unsigned char str)
{
  switch (str)
  {
    case '':
      str = 0xE1;
      break;
    case '':
      str = 0xEF;
      break;
    case '':
      str = 0xF5;
      break;
    case '':
      str = 0xE2;
      break;
    case '':
      str = 0xDF;
      break;
    case '':
      str = 0xE4;
      break;
//    case ':':
//      str = 0x20;
//      break;
  }
  P4OUT=str;                            // str einfach an den port schicken
  P5OUT|=0x05;                          // RS und E high
  P5OUT&=~0x02;                         // R/W low
  LCDClock();                           // befehl ausfuehren
  chars++;                              // zaehlvar. eins erhoehen
/*
  if (chars==ZEICHEN_DISPLAY)           // zeile voll?
  {
    row^=1;                             // toggle row
    chars=0;                            // reset chars
    if (row==1)
    {
      P4OUT=0xC0;                       // in die zweite zeile springen
      P5OUT|=0x04;                          // E high
      P5OUT&=~0x03;                         // RS und R/W low
      LCDClock();                       // befehl ausfuehren
    }
    else
    {
      LCDClear();                       // wieder zurueck in die erste zeile springen
    }
  }
*/
}

void LCDOuts(unsigned char* str)
{
  while (*str != 0) LCDOutc(*str++);
}

void LCDInit(void)
{
  wait(100);                            // display braucht ein wenig zeit nach power-on
  P4OUT=0x38;                           // 8 bit operation, 2-zeilen-display, 5*8 dot font
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
  P4OUT=0x0C;                           // display anschalten, cursor aus, cursor blinkt nicht
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
  P4OUT=0x06;                           // cursorrichtung: nach rechts; kein display-shift
  P5OUT|=0x04;                          // E high
  P5OUT&=~0x03;                         // RS und R/W low
  LCDClock();                           // befehl ausfuehren
}

// writes integer to on LCD
void LCDOutInt ( int value, int digits )
{
  int i;
  int pot = 10;
  unsigned char* str;

  extern char _bss_end;

  _NewHeap(&_bss_end, &_bss_end + 200); // 200 bytes heap
  str = calloc( digits, 4 );
  // have to make 10^digits-1 self, because of stupid compiler behavior whith funkt pot(x,y)
  for ( i = 1; i < digits; i++) {
     pot = pot * 10;
  }

  for ( i = 0; i < digits; i++) {
    str[i] = (value % pot) / (pot / 10) + 0x30;
    if( value % pot == 0 )
       str[i-1] = 0x20;
    pot /= 10;
  }
  str[i+1] = 0x20;

  LCDOuts( str );
}

void LCDOutFloat ( float fl, float digits )
{
  int post;
  int past;
  int pot;
  int i;
  int cnt;

  post = (int)fl;
  LCDOutInt( post, (int)digits);
  LCDOuts(".");

  cnt = (int)(((digits * 10) - ( (int)digits * 10)) /10);

  // must make 10^digits-1 self, because of mad compiler behavior whith funkt pot(x,y)
  for ( i = 1; i < cnt; i++) {
     pot = pot * 10;
  }
  past = (int)((fl - (int)fl) * pot);

  LCDOutInt( past, (int)digits);
}

void IntToString (unsigned char* str, unsigned int value, int digits)
{
  int i;
  long pot = 10;

  // must make 10^digits-1 self, because of mad compiler behavior whith funkt pot(x,y)
  for ( i = 1; i < digits; i++) {
     pot = pot * 10;
  }
//  str[j++] = value / pot + 0x30;
  for ( i = 0; i < digits; i++) {
    str[i] = (value % pot) / (pot / 10) + 0x30;
  pot /= 10;
  }
  str[i] = 0x00;
}
