#include <msp430x14x.h>
#ifndef _lcd_h
  #include "lcd.h"
#endif
void init_Clock( void );
void init_Timer(void);
void main(void);

struct tm
{
  int tm_sec;     /* 0..59 */
  int tm_min;     /* 0..59 */
  int tm_hour;    /* 0..23 */
  int tm_mday;    /* 1..31 */
  int tm_mon;     /* 0..11 */
  int tm_year;    /* 0(:=1900).. */
  int tm_wday;    /* 0..6 */
  int tm_yday;    /* 0..365 */
  int tm_isdst;   /* 0 */
};


unsigned int uiMsCnt;
struct tm stGlobalTime;


void main(void)
{
  char temp_str[20];
  stGlobalTime.tm_year = 2004;
  stGlobalTime.tm_mon  = 11;
  stGlobalTime.tm_mday = 12;
  stGlobalTime.tm_hour = 8;
  stGlobalTime.tm_min  = 14;
  stGlobalTime.tm_sec  = 40;

  WDTCTL = WDTPW + WDTHOLD;             // watchdog aus

  P4DIR  =  0xff;             // port 4 = output (8 data to LCD)

  P5DIR  =  0xff;             // port 5
  P5DIR |=  BIT0;             // port 5 = output LCD RS
  P5DIR |=  BIT1;             // port 5 = output LCD R/W
  P5DIR |=  BIT2;             // port 5 = output LCD E
  P5OUT  =  0x04;             // initial nur E (clock) auf high, RS und R/W auf low


  init_Clock();

  init_Timer();

  _EINT();                              // Enable interrupts

  wait(3000);             // wait for LCD

  LCDInit();              // LCD initialise ...
  LCDCmd( LCD_ON );
  LCDClear();             // clear display and set cursor to first pos
  LCDCmd( LCD_CURSOR_OFF );
  
  while(1) {

  // schriebe zeit auf lcd
    IntToString(temp_str, stGlobalTime.tm_hour, 2);  // int in string umwandeln (mit '.' dazwischen)
    LCDPos(2);
    LCDOuts(temp_str);
    LCDOuts(".");
    IntToString(temp_str, stGlobalTime.tm_min, 2);  // int in string umwandeln (mit '.' dazwischen)
    LCDPos(5);
    LCDOuts(temp_str);
    LCDOuts(".");
    IntToString(temp_str, stGlobalTime.tm_sec, 2);  // int in string umwandeln (mit '.' dazwischen)
    LCDPos(8);
    LCDOuts(temp_str);

  }


}

void init_Clock( void )
{
  int i;
  // Turning XT1 (32kHz) on
  _BIC_SR (OSCOFF);           // Turn on XT1
  BCSCTL1 &= ~XTS;          // LF mode
  for (i = 0xFFFF; i > 0; i--);   // Time for osc. to set
  // XT1 is on

  // Turning XT2 (8MHz) on
  BCSCTL1 &= ~XT2OFF;         // Turn on XT2
  do
  {                       // pruefroutine, ob 8 Mhz-oszillator schon stabil ist
    IFG1 &= ~OFIFG;       // OSCFault flag loeschen (zeigt oszillatorfehler an)
    for (i=0xFF;i>0;i--); // warten, ob es wieder gesetzt wird
  }
  while ((IFG1 & OFIFG) != 0);   // OSCFault immer noch gesetzt?

  BCSCTL2 |=  SELM_2 + DIVM_0 +   // MCLK source is XT2, MCLK div is 1
              SELS + DIVS_0;     // SMCLK souce is DCO, SMCLK div is 1
  BCSCTL1 |= DIVA_0;             // Divider for ACLK is 1

  // At this point we have the following clock configuration:
  // MCLK is 8 MHz (from XT2)
  // SMCLK is 8 MHz (from XT2)
  // ACLK is 32 kHz (from XT1)
}


void init_Timer(void)
{
  TACTL    = TASSEL_1 + TACLR + TAIE; // upcount until CC0 event (usingACLK)
  TACCTL0  = CCIE;
  TACCR0   = 4096;                             // let count 4096 cycles: 125ms intervals
  TACTL   |= MC_2;                             // start timer
}

#ifndef __ICC430
interrupt (TIMERA0_VECTOR) Timer_A0(void)
#else
#pragma interrupt_handler Timer_A0: TIMERA0_VECTOR
void Timer_A0(void)
#endif
{
  TACCR0  += 4096;                      // Add Offset to CCR0 (125ms)
  uiMsCnt += 125;                       // add 125ms
  if( uiMsCnt == 1000 ) {               // 1000ms -&gt; 1sec
    stGlobalTime.tm_sec++;              // increment seconds
    uiMsCnt = 0;                        // reset ms counter
    if( stGlobalTime.tm_sec >= 60 ) {   // 60sec -&gt; 1min
      stGlobalTime.tm_sec = 0;          // reset sec counter
      stGlobalTime.tm_min++;            // increment minute
      if( stGlobalTime.tm_min >= 60 ) { // 60min -&gt; 1hour
        stGlobalTime.tm_min = 0;              // reset mim counter
        stGlobalTime.tm_hour++;               // increment hour
        if( stGlobalTime.tm_hour >= 24 ) {    // 24hour -&gt; 1day
          stGlobalTime.tm_hour = 0;           // reset hour counter
          stGlobalTime.tm_yday++;             // increment day
          if( stGlobalTime.tm_yday >= 365 ) { // 365days -&gt; 1year
            stGlobalTime.tm_yday = 0;         // reset day counter
            stGlobalTime.tm_year++;           // increment year
          }
        }
      }
    }
  }
  TACTL &= ~TAIFG;
}


